"""
load_data.pyでデータベースから取得したデータを予測モデルの入力特徴に変換するプログラム
"""
import os
import pandas as pd
# import argparse
# parser = argparse.ArgumentParser()
# parser.add_argument("--course", type=int, nargs="+", required=True)
# args = parser.parse_args()

# load_data.pyで保存したファイルの親ディレクトリ
SAVE_DIR = "Dataset"
DATASET_DIR = "Dataset"

# 作成した入力特徴を保存するディレクトリ
# DATASET_DIR = "../DATASET"
# os.makedirs(DATASET_DIR, exist_ok=True)

# イベントログをページ遷移ごとに集計する関数．OpenLAに実装されているものと同じ
def convert_into_page_transition(event_stream, invalid_seconds=None, timeout_seconds=None, count_operation=True, operation_name=None):
    # 引数
    #  event_stream (pd.DataFrame)    : イベントログのDataFrame
    #  invalid_seconds (int)          : あるページをinvalid_seconds以上読んでなければ，ページを読んだことにならない
    #  timeout_seconds (int)          : あるページをtimeout_secondsよりも長く読んでいれば，タイムアウトとみなす
    #  count_operation (bool)         : 集計の際に，各ページでの操作（マーカーなど）の回数も集計するかどうか（False: ページ番号と閲覧時間だけ集計，True: 操作回数も集計）
    #  operation_name (str, List[str]): 各操作の回数を集計する場合，どの操作を集計するか（Noneであれば全ての操作を集計）

    def make_empty_df():
        columns = ["user_id", "contents_id", "page_no", "reading_seconds", "time_of_entry", "time_of_exit"]
        if count_operation:
            columns += operation_name
        return pd.DataFrame(columns=columns)

    if operation_name is None:
        operation_name = event_stream["operation_name"].unique()
    elif isinstance(operation_name, str):
        operation_name = [operation_name]

    page_info_dict_list = []
    for ids, user_contents_df in event_stream.groupby(["user_id", "contents_id"], sort=False):
        user_id = ids[0]
        contents_id = ids[1]

        page_seq = user_contents_df["page_no"].tolist()
        operation_seq = user_contents_df["operation_name"].tolist()
        time_seq = pd.to_datetime(user_contents_df["operation_date"]).tolist()

        # aggregation operations in event stream
        operation_count_dict = empty_operation_count = dict(zip(operation_name, [0]*len(operation_name)))
        time_of_entry_index = 0
        reading_seconds = 0

        for i in range(len(page_seq)):
            if count_operation:
                if operation_seq[i] in operation_count_dict:
                    # set new operation as the key
                    operation_count_dict[operation_seq[i]] += 1

            # If (end of sequence) or (close operation is executed) or (page transition will be happened),
            if (i == len(page_seq) - 1) or \
                (operation_seq[i] == "CLOSE") or \
                (i+1 < len(page_seq)) and (page_seq[i] != page_seq[i + 1]):

                time_of_exit_index = i
                time_of_entry = time_seq[time_of_entry_index]
                time_of_exit = time_seq[time_of_exit_index]

                page_info_dict = {"user_id": user_id, "contents_id": contents_id,
                                  "page_no": page_seq[i], "reading_seconds": reading_seconds,
                                  "time_of_entry": time_of_entry, "time_of_exit": time_of_exit}

                # record operation counts in the page
                if count_operation:
                    page_info_dict.update(operation_count_dict)
                # reset operation count dictionary
                operation_count_dict = empty_operation_count

                # if reading seconds is more than invalid seconds, the page is recorded. else it is passed through
                if (invalid_seconds is None) or (reading_seconds > invalid_seconds):
                    page_info_dict_list.append(page_info_dict)
                else:
                    pass
                reading_seconds = 0

                # if operation is 'CLOSE', next operation 'OPEN' is i+1
                if operation_seq[i] == "CLOSE":
                    time_of_entry_index = i + 1
                    continue
                else:
                    time_of_entry_index = i

            # if the time difference between two actions is over timeout_seconds, the event is regarded as time out.
            time_between_actions = (time_seq[i + 1] - time_seq[i]).seconds if i + 1 < len(time_seq) else None
            if time_between_actions is None:
                pass
            elif (timeout_seconds is not None) and (time_between_actions > timeout_seconds):
                time_of_entry_index = i + 1
                pass
            else:
                reading_seconds += time_between_actions

    transition_df = pd.DataFrame(page_info_dict_list)
    transition_df = transition_df.fillna(0)
    if transition_df.empty:
        empty_df = make_empty_df()
        return empty_df
    else:
        return transition_df


# 閲覧時間を取得する関数
def get_reading_time(transition_df, user_id, features_dict):
    # 引数
    #  transition_df (pd.DataFrame)   : convert_into_page_transition()関数で作成されるページ遷移ごとに集計したイベントログ
    #  user_id (str)                  : ユーザーID
    #  features_dict (Dict[str, int]) : 各入力特徴の辞書．key:特徴の名前，value:特徴の値．

    user_df = transition_df[transition_df["user_id"] == user_id]
    if user_df.empty:
        reading_time = 0
    else:
        reading_time = user_df["reading_seconds"].sum()
    features_dict["slide_views"].update({user_id: reading_time})

    return features_dict


# マーカーを引いた数，メモをとった数，全操作の数を取得する関数
def get_operation_counts(event_stream, user_id, features_dict):
    # 引数
    #  event_stream (pd.DataFrame)    : イベントログのDataFrame
    #  user_id (str)                  : ユーザーID
    #  features_dict (Dict[str, int]) : 各入力特徴の辞書．key:特徴の名前，value:特徴の値．

    user_df = event_stream[event_stream["user_id"] == user_id]
    operation_count = user_df["operation_name"].value_counts()
    markers = operation_count["ADD MARKER"] if "ADD MARKER" in operation_count else 0
    memos = operation_count["ADD MEMO"] if "ADD MEMO" in operation_count else 0
    actions = len(user_df)

    features_dict["markers"].update({user_id: markers})
    features_dict["memos"].update({user_id: memos})
    features_dict["actions"].update({user_id: actions})

    return features_dict


# 出席状況を取得する関数
def get_attendance(attendance_df, lecture_week, user_id, features_dict):
    # 引数
    #  attendance_df (pd.DataFrame)   : load_data.pyで取得した出席状況に関するDataFrame
    #  lecture_week (int)             : 出席状況を取得する講義の週
    #  user_id (str)                  : ユーザーID
    #  features_dict (Dict[str, int]) : 各入力特徴の辞書．key:特徴の名前，value:特徴の値．

    if (lecture_week, user_id) not in attendance_df.index:
        value = 0
    else:
        status = attendance_df.xs([lecture_week, user_id], level=["lecture", "userid"])["status"].values[0]
        if status == "Presence":
            value = 5
        elif status == "Late":
            value = 3
        else:
            value = 0
    features_dict["attendance"].update({user_id: value})
    return features_dict


# レポート提出状況を取得する関数
def get_report_submission(report_submission_df, lecture_week, user_id, features_dict):
    # 引数
    #  report_submission_df (pd.DataFrame)   : load_data.pyで取得したレポート提出状況に関するDataFrame
    #  lecture_week (int)                    : レポート提出状況を取得する講義の週
    #  user_id (str)                         : ユーザーID
    #  features_dict (Dict[str, int])        : 各入力特徴の辞書．key:特徴の名前，value:特徴の値．

    value = 0
    if (lecture_week, user_id) not in report_submission_df.index:
        pass
    else:
        submit_status = report_submission_df.xs([lecture_week, user_id], level=["lecture", "userid"])["status"].values
        num_assignments = report_submission_df[list(report_submission_df.reset_index()["lecture"]==lecture_week)]["assignment"].nunique()
        for status in submit_status:
            if status == "Submit":
                value += 5./num_assignments
            elif status == "Late":
                value += 3./num_assignments
            else:
                pass
    features_dict["report"].update({user_id: value})
    return features_dict


#　Moodle上でのコースアクセス回数を取得する関数
def get_course_views(course_views_df, lecture_week, user_id, features_dict):
    # 引数
    #  course_views_df (pd.DataFrame) : load_data.pyで取得したコースアクセス回数に関するDataFrame
    #  lecture_week (int)             : 出席状況を取得する講義の週
    #  user_id (str)                  : ユーザーID
    #  features_dict (Dict[str, int]) : 各入力特徴の辞書．key:特徴の名前，value:特徴の値．

    if (lecture_week, user_id) not in course_views_df.index:
        view_count = 0
    else:
        view_count = course_views_df.xs([lecture_week, user_id], level=["lecture", "userid"])["course_views"].values[0]
    features_dict["course_views"].update({user_id: view_count})
    return features_dict


# すべての特徴をまとめる
def aggregate_feature(course_id):
    lecture_time      = pd.read_csv(f"{SAVE_DIR}/Course_{course_id}_LectureTime.csv")
    event_stream      = pd.read_csv(f"{SAVE_DIR}/Course_{course_id}_EventStream.csv")
    attendance        = pd.read_csv(f"{SAVE_DIR}/Course_{course_id}_Attendance.csv")
    report_submission = pd.read_csv(f"{SAVE_DIR}/Course_{course_id}_ReportSubmission.csv")
    course_views      = pd.read_csv(f"{SAVE_DIR}/Course_{course_id}_CourseViews.csv")

    event_stream["user_id"]     = event_stream["user_id"].apply(str)
    attendance["userid"]        = attendance["userid"].apply(str)
    report_submission["userid"] = report_submission["userid"].apply(str)
    course_views["userid"]      = course_views["userid"].apply(str)

    attendance.set_index(["lecture", "userid"], inplace=True)
    report_submission.set_index(["lecture", "userid"], inplace=True)
    course_views.set_index(["lecture", "userid"], inplace=True)

    grade = pd.read_csv(f"{DATASET_DIR}/grade_{course_id}.csv")
    grade["userid"] = grade["userid"].apply(str)
    users = grade["userid"].tolist()

    features = ["attendance", "report", "course_views", "slide_views", "markers", "memos", "actions"]
    features_df = pd.DataFrame()
    prev_lecture_end = None
    for _, row in lecture_time.iterrows(): # 各講義週について，それぞれ特徴をまとめる
        dicts = [{} for _ in range(len(features))]
        features_dict = dict(zip(features, dicts))
        lecture_week = row["lecture"]
        start_time = row["starttime"]
        end_time = row["endtime"]

        # 講義中のイベントログを抽出
        stream_in_lecture = event_stream[(start_time <= event_stream["operation_date"]) & (event_stream["operation_date"] < end_time)]

        # 講義前の特徴を取得
        if prev_lecture_end is None:
            stream_before_lecture = event_stream[event_stream["operation_date"] < start_time]
        else:
            stream_before_lecture = event_stream[(prev_lecture_end <= event_stream["operation_date"]) & (event_stream["operation_date"] < start_time)]
        transition_df = convert_into_page_transition(stream_before_lecture, invalid_seconds=5, timeout_seconds=30*60, count_operation=False)
        prev_lecture_end = end_time

        #　講義前と講義中のイベントログを結合
        stream = pd.concat([stream_in_lecture, stream_before_lecture])

        # 各ユーザーの特徴を取得
        for user_id in users:
            features_dict = get_reading_time(transition_df=transition_df, user_id=user_id, features_dict=features_dict)
            features_dict = get_operation_counts(event_stream=stream, user_id=user_id, features_dict=features_dict)
            features_dict = get_attendance(attendance_df=attendance, lecture_week=lecture_week, user_id=user_id, features_dict=features_dict)
            features_dict = get_report_submission(report_submission_df=report_submission, lecture_week=lecture_week, user_id=user_id, features_dict=features_dict)
            features_dict = get_course_views(course_views_df=course_views, lecture_week=lecture_week, user_id=user_id, features_dict=features_dict)

        df = pd.DataFrame(features_dict)
        df["classid"] = lecture_week
        features_df = pd.concat([features_df, df], axis=0)
    features_df["userid"] = features_df.index
    features_df.sort_values(["userid", "classid"], inplace=True)
    features_df = features_df.loc[:, ["userid", "classid", "attendance", "report", "course_views", "slide_views", "markers", "memos", "actions"]]

    return features_df


# 各学生の特徴を相対的に評価し，ALP(Active Learner Point)に変換
def relative_evaluation(df, feature_name):
    df = df.sort_values(feature_name, ascending=False)
    df = df.reset_index(drop=True)
    ALP_df = df.copy()
    zero_index = len(df.index)
    for i, value in enumerate(df[feature_name]):
        if value == 0:
            zero_index = i
            break

    upper_10 = round(len(df.index) * 0.1)
    upper_20 = upper_10 * 2
    upper_30 = upper_10 * 3
    upper_40 = upper_10 * 4
    upper_50 = upper_10 * 5

    # 上位何%かによって評価
    ALP_df.loc[0: upper_10, feature_name] = 5

    # 同じ値が範囲外まで続くときは評価値の範囲をそこまで伸ばす
    last_value = df.at[upper_10-1, feature_name]
    for index in range(upper_10, len(df.index)):
        value = df.at[index, feature_name]
        if last_value == value:
            ALP_df.at[index, feature_name] = 5
        else:
            break

    # 上で伸ばした範囲が評価の範囲を飛び越していたらスキップ
    if index < upper_20:
        ALP_df.loc[index: upper_20, feature_name] = 4
        last_value = df.at[upper_20 - 1, feature_name]
        for index in range(upper_20, len(df.index)):
            value = df.at[index, feature_name]
            if last_value == value:
                ALP_df.at[index, feature_name] = 4
            else:
                break

    if index < upper_30:
        ALP_df.loc[upper_20: upper_30, feature_name] = 3
        last_value = df.at[upper_30 - 1, feature_name]
        for index in range(upper_30, len(df.index)):
            value = df.at[index, feature_name]
            if last_value == value:
                ALP_df.at[index, feature_name] = 3
            else:
                break

    if index < upper_40:
        ALP_df.loc[upper_30: upper_40, feature_name] = 2
        last_value = df.at[upper_40 - 1, feature_name]
        for index in range(upper_40, len(df.index)):
            value = df.at[index, feature_name]
            if last_value == value:
                ALP_df.at[index, feature_name] = 2
            else:
                break

    if index < upper_50:
        ALP_df.loc[upper_40: upper_50, feature_name] = 1
        last_value = df.at[upper_50 - 1, feature_name]
        for index in range(upper_50, len(df.index)):
            value = df.at[index, feature_name]
            if last_value == value:
                ALP_df.at[index, feature_name] = 1
            else:
                break

    ALP_df.loc[index:, feature_name] = 0

    # 値が0のところは評価0にする
    ALP_df.loc[zero_index:, feature_name] = 0

    return ALP_df


# 各特徴をALPに変換
def feature2ALP(features_df):
    alp_df = pd.DataFrame()
    # 各週の特徴をALPに変換
    for lecture_week, weekly_df in features_df.groupby("classid"):
        for event in weekly_df.drop(["userid", "attendance", "report", "classid"], axis=1).columns:
            weekly_df = relative_evaluation(df=weekly_df, feature_name=event)
        alp_df = pd.concat([alp_df, weekly_df], axis=0)
    alp_df["userid"] = alp_df["userid"].astype(int)
    alp_df.sort_values(["userid", "classid"], inplace=True)

    features = ["attendance", "report", "course_views", "slide_views", "markers", "memos", "actions"]
    # 0 ~ 1の範囲に正規化
    alp_df_normalized = alp_df.copy()
    alp_df_normalized.loc[:, features] = alp_df_normalized.loc[:, features] / 5

    return alp_df, alp_df_normalized


# def main():
#     for course_id in args.course:
#         print(f"\ncourse id: {course_id}")
# 
#         print("aggregate features...")
#         features_df = aggregate_feature(course_id=course_id)
#         features_df.to_csv(f"{SAVE_DIR}/Course_{course_id}_Feature.csv", index=False)
# 
#         print("convert into ALP...")
#         alp_df, alp_df_normalized = feature2ALP(features_df=features_df)
#         alp_df.to_csv(f"{DATASET_DIR}/original/alp_{course_id}.csv", index=False)
#         alp_df_normalized.to_csv(f"{DATASET_DIR}/alp_{course_id}.csv", index=False)


# if __name__ == "__main__":
#     main()


import numpy as np
import pandas as pd
import datetime as dt
import fasttext as ft
import OpenLA as la
import time
import re

Operation_dict = {
        'NEXT':'N',
        'PREV':'P',
        'ADD MARKER':'A',

        'OPEN':'O',             # 'E'
        'CLOSE':'C',            # 'E'
        'PAGE_JUMP':'J',        # 'E'
        'GETIT':'G',            # 'E'

        'DELETE MARKER':'E',
        'BOOKMARK_JUMP':'E',
        'ADD BOOKMARK':'E',
        'NOTGETIT':'E',
        'ADD MEMO':'E',
        'MEMO_TEXT_CHANGE_HISTORY':'E',

        'DELETE BOOKMARK':'E',
        'CHANGE MEMO':'E',
        'SEARCH_JUMP':'E',
        'REGIST CONTENTS':'E',
        'DELETE_MEMO':'E',
        'SEARCH':'E',
        'OPEN_RECOMMENDATION':'E',
        'CLICK_RECOMMENDATION':'E',
        'TIMER_PAUSE':'E',
        'TIMER_STOP':'E',
        'ADD_HW_MEMO':'E',

        'CLOSE_RECOMMENDATION':'E',
        'CLEAR_HW_MEMO':'E',
        'LINK_CLICK':'E',
        'UNDO_HW_MEMO':'E',
        'ADD_RECOMMENDATION' : 'E',
        'REDO_HW_MEMO' : 'E',
        'DELETE_RECOMMENDATION' : 'E'
        }

class E2Vec(object):
  # 初期化
  def __init__(self, fastText_model, EduData, course_id, SorU='user', dict=Operation_dict , word_time=1):
    self._fT_model = ft.load_model(fastText_model) 
    self._SorU = SorU
    self._dict = Operation_dict
    self._word_time = word_time
    self._course_info = la.CourseInformation(files_dir=EduData , course_id=course_id)
    self._eventstream = self._course_info.load_eventstream() 
    self._all_user_vec =[]
    self._all_sentence_vec = []
    self._back_up = self._course_info
    self._course_id = course_id

  # デバッグ用
  #def _print(self):
  #  print(self._course_info.lecture_start_time(3))

  # オペレーション間の時間を文字化する
  def _interval_check(self, interval_sec:int):
    if interval_sec <= 1:
      interval_word = ''
    elif interval_sec > 1 and interval_sec <= 10:
      interval_word = 's'
    elif interval_sec > 10 and interval_sec <= 300:
      interval_word = 'm'    
    else:
      interval_word = 'l'
    return interval_word

  # 入力useridのイベントストリームを抽出
  # contentsid，eventtimeでsort
  def _get_user_eventstream(self, userid):
    user_stream = la.select_user(self._eventstream, user_id=userid)
    user_stream_df = user_stream.df.sort_values(["contentsid", "eventtime"])
    user_stream_df = user_stream_df[["contentsid", "operationname","eventtime"]]
    df = user_stream_df.replace(self._dict)
    df.index = np.arange(0, len(df))  
    return   df

  # ユーザーごとのイベントストリームからsentenceを生成する
  def _get_oneuser_sentences(self, user_eventstream_df):
  # sentences : output
  # word     : component of sentence
  # current  : event time of current operation
  # previous : event time of previous operation
  # cc       : lecuture materials which the current operation took place
  # pc       : lecuture materials which the previous operation took place
  # start    : Time stamp at the beginning of the word
  # end      : time limit of currentword 
    sentences = ""
    word = ""
    current = dt.datetime.now()
    end = dt.datetime.now()
    cc  = ""
    word_max_len = 15
    for index, data in user_eventstream_df.iterrows():
      if index == 0:                                                             
          current = dt.datetime.strptime((data["eventtime"]), '%Y-%m-%d %H:%M:%S')
          cc = data["contentsid"]
      else:
          pc = cc
          cc = data["contentsid"]
          if pc != cc:
            # contentsidが変化した場合，sentenceを分割
            sentences += (word + '\n')
            word = data['operationname']
            current = dt.datetime.strptime(data["eventtime"], '%Y-%m-%d %H:%M:%S')
            start = current
            end = start + dt.timedelta(minutes=self._word_time)
            continue
          else:
            #contentsidが同じ場合
            #前後のイベントの時間間隔を計算し，該当する文字をwordに追加(_interval_checkを実行)
            previous = current
            current = dt.datetime.strptime(data["eventtime"], '%Y-%m-%d %H:%M:%S')
            interval_sec = current - previous
            interval_sec = interval_sec.seconds
            interval_word = self._interval_check(interval_sec)
            word += interval_word
            # 時間間隔条件によってsentenceを分割する場合の処理
            if interval_word == 'l' :
              sentences += (word + '\n')
              word = data['operationname']
              start = current
              end = start + dt.timedelta(minutes=self._word_time)
              #ログの最後尾以外であればループを継続
              if index != len(user_eventstream_df)-1:
                continue
            # 文字数条件によってsentenceを分割する場合の処理
            elif  len(word) >= word_max_len - 1:
              sentences += (word + '_ ')
              word = data['operationname']
              start = current
              end = start + dt.timedelta(minutes=self._word_time)
              continue
      # ログの最後尾に対する処理
      if index == len(user_eventstream_df)-1:
        if index == 0:
          sentences = data['operationname'] 
        elif current <= end:
          word += data['operationname'] 
          sentences += word
        else:
          sentences += (word + ' ')
          sentences += data['operationname']
      # 先頭のログに対する処理
      elif index == 0:
          start = current
          end = start + dt.timedelta(minutes=self._word_time)
          word = data['operationname']
      # wordの先頭から一定時間以内のログ
      elif current <= end:
          word += data["operationname"]
      # wordの先頭から一定時間以上のログ(新しい単語の先頭になるログ)
      else :
        sentences += (word + ' ')
        word = data["operationname"]
        start = current
        end = start + dt.timedelta(minutes=self._word_time)
    # 終了処理
    if sentences != "":
        return (sentences + '\n')   
  
  # センテンス生成用呼び出しモジュール
  def _get_learninglog_sentences(self, userid):
    user_eventstream_df = self._get_user_eventstream(userid)
    return self._get_oneuser_sentences(user_eventstream_df)

  # ファイル書き出し
  def _write_sentences(self, file_path, sentences):
    f = open(file_path, 'w')
    for sentence in sentences:
      if sentence != None:
        f.write(sentence)
        if self._SorU == 'user':
          f.write('****user****\n')
    f.close()

  # ベクトル書き出し
  def _write_vector(vector: np.ndarray, VectorFilePath):
    vecs_df = pd.DataFrame(vector)
    vecs_df.to_csv(VectorFilePath, index=False)

  #　sentence生成
  def get_sentences(self, file_path):
    usersid = self._eventstream.user_id()
    sentences=[self._get_learninglog_sentences(userid) for userid in usersid]
    self._write_sentences(file_path, sentences)
  # 一時保存先の解放
  def _refresh(self):
    if len(self._all_user_vec) != 0:
      self._all_user_vec = []
    if len(self._all_sentence_vec) != 0:
      self._all_sentence_vec = []  

  # 外部から呼び出し　sentence を入力として vector生成
  def sentences_to_vector(self, sentences_path, save_path, mode='all',start=None, period=None, select_weeks=None, cat_mode=None):
    tmp1 = []
    tmp2 = []
    week = ''
    self._refresh()
    if mode == 'select':
      if select_weeks is None:
        week = 'all'
      else:
        for week_i in select_weeks:
          week += str(week_i)
      #save_path = re.sub('.csv', '_{}-{}_{}.csv'.format(self._start, self._start + self._period, week),save_path) 
    if self._SorU == 'user':
      with open(sentences_path, "r") as f:
        for sentence in f:
          if sentence == '****user****\n':
            if tmp1 != []:
              tmp1 = np.sum(tmp1, axis=0)
              self._all_user_vec.append(tmp1)
              tmp1 = []
            continue
          else:
            sentence = sentence.rstrip('\n')
            tmp2 = self._fT_model.get_sentence_vector(sentence)
            tmp1.append(tmp2)
      self._all_user_vec = np.array(self._all_user_vec)
      vecs_df = pd.DataFrame(self._all_user_vec)
      vecs_df['userid'] = self._index
      vecs_df.set_index('userid', inplace=True)      
    elif self._SorU == 'sentence':
      with open(sentences_path, "r") as f:
        for sentence in f:
          sentence = sentence.rstrip('\n')
          tmp2 =  self._fT_model.get_sentence_vector(sentence)
          self._all_sentence_vec.append(tmp2)
      vecs_df = pd.DataFrame(self._all_sentence_vec)
    if cat_mode == 'time': 
      #vecs_df.to_csv() ある周期のベクトルを保存したい場合
      return vecs_df
    elif cat_mode == 'week':
      #vecs_df.to_csv()　ある周期のベクトルを保存したい場合
      return vecs_df
    else :
      #vecs_df.to_csv(save_path, index=self._index) #save_pathを入力として受け取っておく
      #return save_path                             pathを返したい場合
      return vecs_df
    return vecs_df #DataFrameを返すのみの場合，if cat_mode 以下をなくしてここを実行すれば良い
    

  # 指定したタイミングの時間を計算して開始時間と終了時間を返す
  def _specific_time_extraction(self, start:int, period:int):
    start_end = []
    for i in range(len(self._course_info.lecture_time_df())):
      lecture_start_time = self._course_info.lecture_start_time(i+1) + dt.timedelta(minutes=start) 
      lecture_end_time = lecture_start_time + dt.timedelta(minutes=period)
      start_end.append([lecture_start_time,lecture_end_time])
    return start_end

  # 特定の時間のイベントストリームを取得
  def _specifictime_eventstream(self, start_end:list, eventstream_path, select_weeks=None):
    if select_weeks == None:
      select_weeks = [i+1 for i in range(len(start_end))]
    select_time_df = pd.DataFrame()
    for lecture_week in select_weeks:
      selected_time = start_end[lecture_week - 1]
      one_select_time_df = la.data_extraction.select_time(self._eventstream, start_time=selected_time[0], end_time=selected_time[1]).df
      select_time_df = pd.concat([select_time_df, one_select_time_df])
    select_time_df.to_csv(eventstream_path, index=False)

 # イベントストリーム置き換えモジュール(時間指定の時などに使用)
  def _replace_eventstream(self, eventstream_path, info_dir, course_id):
    self._course_info = la.CourseInformation(event_stream_file    = eventstream_path,
                                        lecture_material_file= info_dir + "/Course_{}_LectureMaterial.csv".format(course_id),
                                        lecture_time_file    = info_dir + "/Course_{}_LectureTime.csv".format(course_id),
                                        grade_point_file     = info_dir + "/Course_{}_GradePoint.csv".format(course_id))
    self._eventstream = self._course_info.load_eventstream()
    print(len(self._eventstream.df))

  # 外部から呼び出すモジュール 指定した条件に合うテキストファイル(sentences)を作成し，そのパスを出力とする．
  def get_Sentences(self, sentences_path, eventstream_path, info_dir, course_id, mode='all', start=0, period=90, select_weeks=None):
    if mode == 'all':
      pass
    elif mode == 'select':
      #sl_time0 = time.time()
      self._start = start
      self._period = period
      selcted_time = self._specific_time_extraction(start, period)
      self._specifictime_eventstream(selcted_time, eventstream_path, select_weeks)
      self._replace_eventstream(eventstream_path, info_dir, course_id)
      sentences_path_tmp = re.sub('.txt', '{}-{}.txt'.format(start, period+start),sentences_path) 
      #sl_time1 = time.time()
      #print("select time log " + str(sl_time1 - sl_time0) + " seconds")
    usersid = self._eventstream.user_id()
    self._index = usersid
    #l2s_time0 = time.time()
    sentences=[self._get_learninglog_sentences(userid) for userid in usersid]
    #l2s_time1 = time.time()
    #print("log to sentences " + str(l2s_time1 - l2s_time0) + " seconds")
    if mode == 'all':
      self._write_sentences(sentences_path, sentences)
      return sentences_path
    elif mode == 'select':
      self._write_sentences(sentences_path_tmp, sentences)
      # 繰り返しの実行のために限定した設定の初期化
      self._course_info = self._back_up
      self._eventstream = self._course_info.load_eventstream() 
      return sentences_path_tmp

  # 時間ごとに集約したベクトルを連結して出力
  def get_concat_vectors(self, sentences_path, eventstream_path, vector_path ,info_dir, course_id, concat_mode='time', start=0, period=90,  select_weeks=None):
    vec_df = pd.DataFrame()
    sub_vec_df = pd.DataFrame()
    lecture_time = self._course_info.lecture_end_time(1)-self._course_info.lecture_start_time(1)
    lecture_time_minutes = int(lecture_time.seconds/60)
    if concat_mode == 'time':
      for i in range(int(lecture_time_minutes/period)):
        #ex0 = time.time()
        sentences_path_tmp = self.get_Sentences(sentences_path,mode='select', start=start, period=period, eventstream_path=eventstream_path, info_dir=info_dir, course_id=course_id, select_weeks=select_weeks)
        #ex1 = time.time()
        sub_vec_df = self.sentences_to_vector(sentences_path_tmp, vector_path, mode='select', start=start, period=period, select_weeks=select_weeks, cat_mode=concat_mode)
        ex2 = time.time()
        #print('senetence '+ str(ex1-ex0) +' seconds')
        #print('vecotors '+ str(ex2-ex1) +' seconds')
        sub_vec_df['userid'] = self._index
        sub_vec_df.set_index('userid', inplace=True)
        self._course_info = self._back_up
        self._eventstream = self._course_info.load_eventstream() 
        if sub_vec_df.empty:
          continue
        sub_vec_df.columns=[self._fT_model.get_dimension()*i + j for j in range(self._fT_model.get_dimension())]
        vec_df = pd.merge(vec_df, sub_vec_df, left_index=True, right_index=True, how='outer')
        start = start + period
        # user id を基準にconcatする必要がある
    elif concat_mode == 'week':
      num_of_lecture = len(self._course_info.lecture_time_df())
      for i in range(num_of_lecture):
        one_week = [i+1]
        sentences_path_tmp = self.get_Sentences(sentences_path,mode='select', start=start, period=lecture_time_minutes, eventstream_path=eventstream_path, info_dir=info_dir, course_id=course_id, select_weeks=one_week)
        sub_vec_df = self.sentences_to_vector(sentences_path_tmp, save_path=vector_path, mode='select', start=0, period=lecture_time_minutes, select_weeks=one_week, cat_mode=concat_mode)
        self._course_info = self._back_up
        self._eventstream = self._course_info.load_eventstream()
        if sub_vec_df.empty:
          print("eventstream in lecture {} is empty.".format(one_week[0]))
          continue
        sub_vec_df['userid'] = self._index
        sub_vec_df.set_index('userid', inplace=True)
        #sub_vec_df.to_csv(r"C:\Users\miyazakiyuma\code\python\E2Vec\Vectors\Test-UserVectors_D-2022_1min_{}.csv".format(one_week))
        sub_vec_df.columns=[self._fT_model.get_dimension()*i + j for j in range(self._fT_model.get_dimension())]
        vec_df = pd.merge(vec_df, sub_vec_df, left_index=True, right_index=True, how='outer') 
    #ここまでを終えて，ログがなかった部分はNaNが入っていてそのままでは計算できないので，NaNを0に置換する
    vec_df = vec_df.fillna(0)
    #vector_path_con = re.sub('.csv', '_{}.csv'.format(concat_mode),vector_path)
    #vec_df.to_csv(vector_path_con)
    return vec_df
    #return vector_path_con


import numpy as np
import pandas as pd
import OpenLA as la


# 閲覧時間と閲覧ページ数(読み返し等の重複含む)の取得
def read_time_and_page(event_stream):

  # ページ遷移ごとの集計
  page_transition = la.convert_into_page_transition(event_stream,          # EventStreamクラスのインスタンス
                                                    user_id=None,          # 変換に含めるユーザーID，リストによる複数指定も可．None: すべて含める
                                                    contents_id=None,      # 変換に含めるコンテンツID，リストによる複数指定も可．None: すべて含める
                                                    invalid_seconds=0,     # 指定した秒数以下の時間で遷移したページは見ていないものとして無効とされる
                                                    timeout_seconds=20*60, # 指定した秒数以上の時間何も操作がなければタイムアウトとして無効とされる
                                                    count_operation=False, # ページ遷移ごとの操作回数も集計するか(True)，ページ遷移ごとの閲覧時間のみ集計するか(False)
                                                    operation_name=None)   # count_operation=Trueの場合，集計に含める操作の名前．None: すべて含める

  # 集計結果から，各ユーザーの閲覧時間と閲覧ページ数を取得（すべてのコンテンツの合計）
  user_list = []
  num_read_pages_list = []
  reading_seconds_list = []
  for user in page_transition.user_id():

    # Page Transitionクラスのnum_transitionメソッドにより遷移回数（重複含む閲覧ページ数）を取得
    # user_id変数で取得するユーザーを指定．contents_id=Noneですべてのコンテンツに対する合計閲覧ページ数を取得
    num_read_pages  = page_transition.num_transition(user_id=user, contents_id=None) 

    # Page Transitionクラスのreading_timeメソッドにより閲覧時間を取得
    # user_id変数で取得するユーザーを指定．contents_id=Noneですべてのコンテンツに対する合計閲覧時間を取得
    reading_seconds = page_transition.reading_time(time_unit="seconds", user_id=user, contents_id=None)

    user_list.append(user)
    num_read_pages_list.append(num_read_pages)
    reading_seconds_list.append(reading_seconds)

  # データフレーム形式にまとめる
  time_page_df = pd.DataFrame({"userid": user_list, 
                               "num_read_pages": num_read_pages_list, 
                               "reading_seconds": reading_seconds_list})
  return time_page_df


# 特徴量作成
def make_feature(event_stream):

  # 閲覧時間と閲覧ページの数を先ほど定義した関数で取得
  time_page_df = read_time_and_page(event_stream)

  # 操作回数の集計
  operation_count = la.convert_into_operation_count(event_stream,           # EventStreamクラスのインスタンス
                                                    user_id=None,           # 変換に含めるユーザーID，リストによる複数指定も可．None: すべて含める
                                                    contents_id=None,       # 変換に含めるコンテンツID，リストによる複数指定も可．None: すべて含める
                                                    operation_name=None,    # 変換に含める操作の名前．None: すべて含める
                                                    for_each_content=False) # コンテンツごとに操作回数を集計するか(True)，すべての合計を出すか(False)

  # 閲覧時間，閲覧ページ数と各操作回数のデータフレームを結合して特徴量とする
  feature_df = pd.merge(left=time_page_df, right=operation_count.df, on="userid")
  feature_df = feature_df.fillna(0)
  feature_df.set_index("userid", drop=True, inplace=True)

  return feature_df



import numpy as np
import pandas as pd
import tensorflow as tf
from sklearn.model_selection import StratifiedKFold


# 成績を出力のインデックスに変換（0: no-risk, 1:at-risk）
def change_grade(grade):
    if grade == "A":
        grade = 0
    elif grade == "B":
        grade = 0
    elif grade == "C":
        grade = 1
    elif grade == "D":
        grade = 1
    elif grade == "F":
        grade = 1
    else:
        raise ValueError("invalid grade")

    return grade


# sourceデータとtargetデータの入力特徴と成績のデータをDataFrameで返す
def load_source_target(source_course, target_course=None, until_week=None):
    # 引数
    #  source_course (List[int] or int): source courseの ID．
    #  target_course (List[int] or int): target courseの ID. sourceデータでクロスバリデーションをするときは関係ない
    #  until_week (int)　　　　　　　　　　　　　　: 何週目までのデータを入力するか

    # リスト型での処理を行うため，使用するコースが複数でないならばリストに変換
    if type(source_course) is int:
        source_course = [source_course]
    if type(target_course) is int:
        target_course = [target_course]

    # データセットをDataFrameで読み込み
    def load_df(courses):
        all_score_df, all_feature_df = pd.DataFrame(), pd.DataFrame()
        for course in courses:
            score_df = pd.read_csv(f"Dataset/grade_{course}.csv")
            feature_df = pd.read_csv(f"Dataset/alp_{course}.csv")

            score_df["userid"] = score_df["userid"].apply(lambda id: str(course) + "_" + str(id))
            feature_df["userid"] = feature_df["userid"].apply(lambda id: str(course) + "_" + str(id))

            all_score_df = pd.concat([all_score_df, score_df], axis=0)
            all_feature_df = pd.concat([all_feature_df, feature_df], axis=0)

        return all_score_df, all_feature_df
    source_score_df, source_feature_df = load_df(courses=source_course)
    target_score_df, target_feature_df = load_df(courses=target_course)

    source_score_df.set_index("userid", inplace=True)
    target_score_df.set_index("userid", inplace=True)

    # 成績(A,B,C,D,F)を出力のインデックスに変換（0: no-risk, 1:at-risk）
    source_score_df["grade"] = source_score_df["grade"].apply(change_grade)
    target_score_df["grade"] = target_score_df["grade"].apply(change_grade)

    # モデルの訓練/評価に使う週までのデータのみを抽出
    source_feature_df = source_feature_df[source_feature_df["classid"].isin(list(range(1, until_week+1)))]
    target_feature_df = target_feature_df[target_feature_df["classid"].isin(list(range(1, until_week+1)))]

    # 戻り値
    # source_feature_df (pd.DataFrame) : モデル訓練に使う入力特徴のDataFrame
    # source_score_df (pd.DataFrame)   : モデル訓練に使う正解ラベルのDataFrame
    # target_feature_df (pd.DataFrame) : モデル評価に使う入力特徴のDataFrame
    # target_score_df (pd.DataFrame)   : モデル評価に使う正解ラベルのDataFrame
    return source_feature_df, source_score_df, target_feature_df, target_score_df


# DataFrameから必要な情報を取り出してモデルへの入出力に変換する
def input_output(feature_df, score_df, return_sequences):
    # 引数
    # source_feature_df (pd.DataFrame) : 入力特徴のDataFrame
    # source_score_df (pd.DataFrame)   : 正解ラベルのDataFrame
    # return_sequences (bool)          : すべての時系列に対して出力を行うかどうか
    x = []
    y = []
    output_units = 2
    for user_id, user_df in feature_df.groupby("userid"):
        if user_id not in score_df.index:
            continue
        user_df.sort_values("classid", inplace=True) # 時系列順に並べる

        score = score_df.at[user_id, "grade"]
        if return_sequences:
            score = [score] * len(user_df)  # 全ての時系列に与えるため，時系列長だけ複製

        x.append(user_df.drop(["userid", "classid"], axis=1).values) # 不要なカラムを削除して入力に追加
        y.append(np.eye(output_units)[score])                          # one-hotベクトル化して追加
    x = np.array(x).astype(np.float32)
    y = np.array(y)

    return x, y

# DataFrameから訓練時と評価時に使う入力と出力を作る
def make_input_output(source_feature_df, source_score_df, target_feature_df, target_score_df, return_sequences=True):
    # 引数
    # source_feature_df (pd.DataFrame)        : モデル訓練に使う入力特徴のDataFrame
    # source_score_df (pd.DataFrame)          : モデル訓練に使う正解ラベルのDataFrame
    # target_feature_df (pd.DataFrame)        : モデル評価に使う入力特徴のDataFrame
    # target_score_df (pd.DataFrame)          : モデル評価に使う正解ラベルのDataFrame
    # return_sequences (bool)                 : すべての時系列に対して出力を行うかどうか

    x_train, y_train = input_output(source_feature_df, source_score_df, return_sequences)
    x_test, y_test = input_output(target_feature_df, target_score_df, return_sequences)
    timesteps = x_train.shape[1]
    num_features = x_train.shape[-1]

    # 戻り値
    # x_train (np.array): モデル訓練に使う入力．shape:(ユーザー数，時系列長，特徴数)
    # y_train (np.array): モデル訓練に使う出力．shape:(ユーザー数，正解ラベルの次元数(2))
    # x_test  (np.array): モデル評価に使う入力．shape:(ユーザー数，時系列長，特徴数)
    # y_test  (np.array): モデル評価に使う出力．shape:(ユーザー数，正解ラベルの次元数(2))
    # timesteps (int)   : 時系列の長さ
    # num_features (int): 入力特徴の数
    return x_train, y_train, x_test, y_test, timesteps, num_features



# DataFrameからKnowledge distillationを使った訓練と評価のための入力と出力を作る
def make_input_output_kd(source_feature_df, source_score_df, target_feature_df, target_score_df,
                         return_sequences=True, until_week=None,
                         teacher_model=None, source_feature_df_teacher=None,):
    # 引数
    # source_feature_df (pd.DataFrame)        : モデル訓練に使う入力特徴のDataFrame
    # source_score_df (pd.DataFrame)          : モデル訓練に使う正解ラベルのDataFrame
    # target_feature_df (pd.DataFrame)        : モデル評価に使う入力特徴のDataFrame
    # target_score_df (pd.DataFrame)          : モデル評価に使う正解ラベルのDataFrame
    # return_sequences (bool)                 : すべての時系列に対して出力を行うかどうか
    # kd (bool)                               : Knowledge Distillation (KD) 用の入出力を作るかどうか
    # teacher_model (RNN)                     : KDの教師モデル
    # source_feature_df_teacher (pd.DataFrame): KDの教師モデルの入力特徴
    # until_week (int):                       : 何週目までを入力して学習するか

    def input_output_kd(feature_df_student, feature_df_teacher, score_df, teacher_model):
        x = []
        y_true = []
        y_pred = []
        y_hidden = []
        output_units = 2
        for user_id, student_feature in feature_df_student.groupby("userid"):
            if user_id not in score_df.index: # 成績が無いやつは除外
                continue
            student_feature.sort_values("classid", inplace=True) # 生徒モデルに入力する特徴を時系列順に並べる

            # 生徒モデルに入力したのと同じ学生についての教師モデル用の入力特徴を取得
            teacher_feature = feature_df_teacher[feature_df_teacher["userid"] == user_id]
            teacher_feature.sort_values("classid", inplace=True)
            teacher_feature = np.expand_dims(teacher_feature.drop(["userid", "classid"], axis=1).values, axis=0)

            # 教師モデルの時系列最後の予測結果を取得
            if return_sequences:
                pred_score, final_hidden = teacher_model.predict(teacher_feature)
                pred_score = pred_score[0][-1]
            else:
                pred_score, final_hidden = teacher_model.predict(teacher_feature)
            # 教師モデルの時系列最後の隠れ状態を取得
            final_hidden = final_hidden.reshape(final_hidden.shape[-1])

            # 正解ラベルを取得
            score = score_df.at[user_id, "grade"]
            if return_sequences:
                score = [score] * until_week

            x.append(student_feature.drop(["userid", "classid"], axis=1).values)
            y_true.append(np.eye(output_units)[score])
            y_pred.append(pred_score)
            y_hidden.append(final_hidden)

        x = np.array(x).astype(np.float32)
        y_true = np.array(y_true)
        y_pred = np.array(y_pred)
        y_hidden = np.array(y_hidden)
        return x, y_true, y_pred, y_hidden

    x_train, y_true, y_pred, y_hidden = input_output_kd(source_feature_df, source_feature_df_teacher, source_score_df, teacher_model)
    x_test, y_test = input_output(target_feature_df, target_score_df, return_sequences)
    timestep = 1
    num_features = x_train.shape[-1]

    # 戻り値
    #  x_train, y_true, x_test, y_test, timestep, num_features : make_input_output()関数と同じ
    #  y_pred (np.array)   : 教師モデルの時系列最後の出力．shape:(ユーザー数，正解ラベルの次元数(2))
    #  y_hidden (np.array) : 教師モデルの時系列最後の隠れ状態．shape:(ユーザー数，隠れ層のユニット数)
    return x_train, y_true, y_pred, y_hidden, x_test, y_test, timestep, num_features


def load_dataset_baseline(source_course, target_course=None, return_sequences=True, until_week=None):
    # 入力特徴と出力のDataFrameを取得
    source_target = load_source_target(source_course=source_course, target_course=target_course, until_week=until_week)
    source_feature_df, source_score_df, target_feature_df, target_score_df = source_target

    # DataFrameからモデルの入出力データを取得
    input_output = make_input_output(source_feature_df, source_score_df, target_feature_df, target_score_df, return_sequences)
    x_train, y_train, x_test, y_test, timestep, num_features = input_output

    return x_train, y_train, x_test, y_test, timestep, num_features


def load_dataset_kd(source_course, teacher_model: tf.keras.Model, target_course=None,
                    return_sequences=True, until_week=None, teacher_week=None):

    # 教師モデルの入力特徴と出力のDataFrameを取得
    source_target_teacher = load_source_target(source_course=source_course, target_course=target_course, until_week=teacher_week)
    source_feature_df_teacher, source_score_df, _, _ = source_target_teacher

    # 生徒モデルの入力特徴と出力のDataFrameを取得
    source_target_student = load_source_target(source_course=source_course, target_course=target_course, until_week=until_week)
    source_feature_df_student, _, target_feature_df, target_score_df = source_target_student

    # DataFrameから入出力データを取得
    input_output = make_input_output_kd(source_feature_df_student, source_score_df, target_feature_df, target_score_df,
                                        return_sequences=return_sequences, teacher_model=teacher_model,
                                        source_feature_df_teacher=source_feature_df_teacher, until_week=until_week)
    x_train, y_true, y_pred, y_hidden, x_test, y_test, timestep, num_features = input_output

    return x_train, y_true, y_pred, y_hidden, x_test, y_test, num_features




import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from sklearn.metrics import confusion_matrix

def import_source_data(path, filename):
    data = pd.read_csv('{}/{}.csv'.format(path, filename), index_col=False)
    data_columns = list(data.columns)
    if 'lecture' in data_columns:
        data = data.sort_values(by=['userid', 'lecture']).reset_index(drop=True)
    elif 'classid' in data_columns:
        data = data.sort_values(by=['userid', 'classid']).reset_index(drop=True)
    data = data.fillna(0)
    return data


def grade_to_class(data):
    data = data.sort_values(by=['userid']).reset_index(drop=True)
    data_grade = list(data['grade'])
    grade_classification = list()
    
    for i in range(len(data_grade)):
        if data_grade[i] == 'A':
            grade_classification.append(0)
        elif data_grade[i] == 'B':
            grade_classification.append(0)
        elif data_grade[i] == 'C':
            grade_classification.append(1)
        elif data_grade[i] == 'D':
            grade_classification.append(1)
        else:
            grade_classification.append(1)
    grade_classification_ = np.array(grade_classification)
    onehot_array = np.zeros((grade_classification_.size, grade_classification_.max() + 1))
    onehot_array[np.arange(grade_classification_.size), grade_classification_] = 1
    return onehot_array, grade_classification_


def precision_recall_fscore_accuracy_binary(confusion_matrix_value):
    TP = confusion_matrix_value[3]
    FP = confusion_matrix_value[1] 
    TN = confusion_matrix_value[0] 
    FN = confusion_matrix_value[2]
    # print(TP, FN, FP, TN)

    precision = float(TP / (TP + FP))
    recall =  float(TP / (TP + FN))
    f_score =  float((2 * precision * recall) / (precision + recall))
    accuracy =  float((TP + TN) / (TP + FN + TN + FP))

    # print("Precision: {:.3f},  Recall: {:.3f}, F-score: {:.3f}, Accuracy: {:.3f}".format(precision, recall, f_score, accuracy))
    return precision, recall, f_score, accuracy


def no_and_at_risk_ordes(labels_list):
    no_risk_orders = list()
    at_risk_orders = list()
    for i in range(len(labels_list)):
        if labels_list[i] == 0:
            no_risk_orders.append(i)
        else:
            at_risk_orders.append(i)
    return no_risk_orders, at_risk_orders


def a_b_c_d_f_orders(grade_labels):
    a_orders = list()
    b_orders = list()
    c_orders = list()
    d_orders = list()
    f_orders = list()
    for i in range(len(grade_labels['grade'])):
        if grade_labels['grade'][i] == 'A':
            a_orders.append(i)
        elif grade_labels['grade'][i] == 'B':
            b_orders.append(i)
        elif grade_labels['grade'][i] == 'C':
            c_orders.append(i)
        elif grade_labels['grade'][i] == 'D':
            d_orders.append(i)
        else:
            f_orders.append(i)
    return a_orders, b_orders, c_orders, d_orders, f_orders


def time_features_attention_calculation(list_orders, attention_time_layer_output_, attention_features_layer_output_):
    attention_time_stu = list()
    attention_features_stu = list()
    for train_stu_order in list_orders:
        attention_time = attention_time_layer_output_[train_stu_order]
        attention_features = attention_features_layer_output_[train_stu_order]
        attention_time_stu.append(attention_time)
        attention_features_stu.append(attention_features)

    attention_time_stu_ = np.array(attention_time_stu)
    attention_features_stu_ = np.array(attention_features_stu)

    attention_time_stu_mean = np.round(np.mean(attention_time_stu_, axis=0, dtype=np.float32), decimals= 3)
    attention_features_stu_mean = np.round(np.mean(attention_features_stu_, axis=0, dtype=np.float32), decimals= 3)
    attention_time_stu_mean = attention_time_stu_mean.reshape(attention_time_stu_mean.shape[1])
    
    attention_time_stu_std = np.round(np.std(attention_time_stu_, axis=0, dtype=np.float32), decimals= 3)
    attention_features_stu_std = np.round(np.std(attention_features_stu_, axis=0, dtype=np.float32), decimals= 3)
    attention_time_stu_std = attention_time_stu_std.reshape(attention_time_stu_std.shape[1])
    return attention_time_stu_mean, attention_time_stu_std, attention_features_stu_mean, attention_features_stu_std


def attention_data(Y_test, attention_time_layer, attention_feature_layer):
    all_stu_time = list()
    all_stu_feature = list()
    for i in range(Y_test.shape[0]):
        attention_time_stu, _, attention_features_stu, _ = time_features_attention_calculation([i], attention_time_layer, attention_feature_layer)
        all_stu_time.append(attention_time_stu)
        attention_features_stu_ = attention_features_stu.flatten()
        all_stu_feature.append(attention_features_stu_)

    all_stu_time_ = np.array(all_stu_time)
    all_stu_feature_ = np.array(all_stu_feature)

    attention_time = pd.DataFrame(all_stu_time_)
    attention_feature = pd.DataFrame(all_stu_feature_)
    return attention_time, attention_feature


def confusion_matrix_calculator(true_list, predict_result_list):
    cm  = confusion_matrix(true_list, predict_result_list)
    fig, ax = plt.subplots(figsize=(5, 5))
    ax.imshow(cm)
    ax.grid(False)
    ax.xaxis.set(ticks=(0, 1), ticklabels=('Predict NR', 'Predict AR'))
    ax.yaxis.set(ticks=(0, 1), ticklabels=('Actual NR', 'Actual AR'))
    confusion_matrix_value = list()
    for i in range(2):
        for j in range(2):
            ax.text(j, i, cm[i, j], ha='center', va='center', color='red')
            confusion_matrix_value.append(cm[i, j])
    plt.show()
    return confusion_matrix_value
